unit uPopClient;

interface

uses  Windows
    , Classes
    , Pop3Prot
    , Wsocket
    ;

type
    POP3_Message = class

    private

        FSource         : TPop3Cli;
        FMsgNumber      : Integer;
        FMsgSize        : Integer;
        FHeaders        : TStringList;
        FLastKey        : String;

        function GetFrom : String;
        function GetSubject : String;
        function GetHeader( name : String ) : String;
        function GetAllHeaders : TStrings;

    public
        constructor Create( source : TPop3Cli );
        destructor Destroy; override;

        property MsgNumber : Integer
            read FMsgNumber;

        property MsgSize : Integer
            read FMsgSize;

        property AllHeaders : TStrings
            read GetAllHeaders;

        property From : String
            read GetFrom;

        property Subject : String
            read GetSubject;

        property Header[ name : String ] : String
            read GetHeader;

        procedure ParseHeaderLine( line : String );
    end;

    POP3_FSM = class

    private

        FStateStack     : TList;
        FClient         : TPOP3Cli;

    public

        constructor Create( p3 : TPop3Cli );
        destructor Destroy; override;

        procedure Push( p : Pointer );
        function Pop : Pointer;
    end;

implementation

uses  SysUtils
    ;

////////////////////////////////////////////////////////////////////////////////
// POP3_Message
////////////////////////////////////////////////////////////////////////////////

constructor POP3_Message.Create( source : TPop3Cli );
begin
    inherited Create;

    FSource         := Source;
    FHeaders        := TStringList.Create;

    with source do
    begin
        FMsgNumber  := MsgNum;
        FMsgSize    := MsgSize;
    end;
end;


destructor POP3_Message.Destroy;
begin
    FHeaders.Free;
    inherited Destroy;
end;



procedure ParseHeader( header       : String
                     ; headers      : TStrings
                     ; var lastKey  : String
                     );
var
    p       : PChar;
    q       : PChar;
    key     : String;
    value   : String;

begin
    p   := PChar( header );
    q   := p;
    if p^ in [ #9, ' ' ] then
    begin
        headers.Values[ lastKey ] := headers.Values[ lastKey ]
                                   + #10#13 + header;
        Exit;
    end;
    while p^ <> #0 do
    begin
        if p^ = ':' then
        begin
            key     := Copy( header, 1, p - q );
            value   := Copy( header, p - q + 2, 9999 );
            if headers.Values[ key ] <> '' then
                headers.Values[ key ] := headers.Values[ key ] + #10#13 + value
            else
                headers.Values[ key ] := value;
            lastKey := key;
            break;
        end
        else
            Inc( p );
    end;
end;


procedure POP3_Message.ParseHeaderLine( line : String );
begin
    ParseHeader( line, FHeaders, FLastKey );
end;


function POP3_Message.GetAllHeaders : TStrings;
begin
    Result := FHeaders;
end;


function POP3_Message.GetFrom : String;
begin
    Result := Header[ 'From' ];
end;


function POP3_Message.GetSubject : String;
begin
    Result := Header[ 'Subject' ];
end;


function POP3_Message.GetHeader( name : String ) : String;
begin
    Result := FHeaders.Values[ name ];
end;


////////////////////////////////////////////////////////////////////////////////
//  POP3_FSM
////////////////////////////////////////////////////////////////////////////////

type
    POP3_HandlerMemo = class
        FMsgNum             : Integer;
        FOnDisplay          : TPop3Display;
        FOnMessageBegin     : TNotifyEvent;
        FOnMessageEnd       : TNotifyEvent;
        FOnMessageLine      : TNotifyEvent;
        FOnListBegin        : TNotifyEvent;
        FOnListEnd          : TNotifyEvent;
        FOnListLine         : TNotifyEvent;
        FOnUidlBegin        : TNotifyEvent;
        FOnUidlEnd          : TNotifyEvent;
        FOnUidlLine         : TNotifyEvent;
        FOnStateChange      : TNotifyEvent;
        FOnRequestDone      : TPop3RequestDone;
        FOnResponse         : TPop3Display;
        FOnSessionConnected : TSessionConnected;
        FOnSessionClosed    : TSessionClosed;
        FPointer            : Pointer;
    end;

procedure POP3_FSM.Push( p : Pointer );
var
    memo    : POP3_HandlerMemo;

begin
    memo                := POP3_HandlerMemo.Create;
    with memo do
    begin                    
        FMsgNum             := FClient.MsgNum;
        FOnDisplay          := FClient.OnDisplay;
        FOnMessageBegin     := FClient.OnMessageBegin;
        FOnMessageEnd       := FClient.OnMessageEnd;
        FOnMessageLine      := FClient.OnMessageLine;
        FOnListBegin        := FClient.OnListBegin;
        FOnListEnd          := FClient.OnListEnd;
        FOnListLine         := FClient.OnListLine;
        FOnUidlBegin        := FClient.OnUidlBegin;
        FOnUidlEnd          := FClient.OnUidlEnd;
        FOnUidlLine         := FClient.OnUidlLine;
        FOnStateChange      := FClient.OnStateChange;
        FOnRequestDone      := FClient.OnRequestDone;
        FOnResponse         := FClient.OnResponse;
        FOnSessionConnected := FClient.OnSessionConnected;
        FOnSessionClosed    := FClient.OnSessionClosed;
        FPointer            := p;
    end;
    FStateStack.Add( memo );
end;


procedure POP3_FSM.Pop;
var
    memo    : POP3_HandlerMemo;

begin
    with FStateStack do
    begin
        memo                := POP3_HandlerMemo( Items[ Count - 1 ] );
        Items[ Count - 1 ]  := Nil;
        Pack;
    end;

    with memo do
    begin                                     
        FClient.MsgNum              := FMsgNum;
        FClient.OnDisplay           := FOnDisplay;
        FClient.OnMessageBegin      := FOnMessageBegin;
        FClient.OnMessageEnd        := FOnMessageEnd;
        FClient.OnMessageLine       := FOnMessageLine;
        FClient.OnListBegin         := FOnListBegin;
        FClient.OnListEnd           := FOnListEnd;
        FClient.OnListLine          := FOnListLine;
        FClient.OnUidlBegin         := FOnUidlBegin;
        FClient.OnUidlEnd           := FOnUidlEnd;
        FClient.OnUidlLine          := FOnUidlLine;
        FClient.OnStateChange       := FOnStateChange;
        FClient.OnRequestDone       := FOnRequestDone;
        FClient.OnResponse          := FOnResponse;
        FClient.OnSessionConnected  := FOnSessionConnected;
        FClient.OnSessionClosed     := FOnSessionClosed;;
        Result                      := FPointer;
        Free;
    end;
end;


constructor POP3_FSM.Create( p3 : TPop3Cli );
begin
    inherited Create;
    FStateStack     := TList.Create;
    FClient         := p3;
    Push( Nil );
end;


destructor POP3_FSM.Destroy;
begin
    Pop;
    inherited Destroy;
end;


end.
